"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.divideConvertBackSmallestUnit = exports.multiplyByDecimals = exports.convertBack = exports.ensureDecimal = exports.priceFormatter = void 0;
const currency_js_1 = __importDefault(require("currency.js"));
const decimal_js_1 = __importDefault(require("decimal.js"));
/**
 * Formats a given price according to the specified currency details and formatting rules,
 * optionally converting the price from a source currency to a target currency, while removing trailing zeros.
 *
 * @param {Object} params - The parameters for the price formatter.
 * @param {number} params.price - The price to be formatted.
 * @param {boolean} [params.fromSmallestUnit=false] - Indicates whether the price is provided in the smallest unit of the source currency.
 * @param {Object} params.fromCurrency - The source currency properties.
 * @param {string} params.fromCurrency.symbol - The symbol of the source currency.
 * @param {number} params.fromCurrency.decimals - The number of decimal places for the source currency.
 * @param {Object} [params.toCurrency] - The target currency properties. If provided, the price is converted to this currency.
 * @param {string} params.toCurrency.symbol - The symbol of the target currency.
 * @param {number} params.toCurrency.decimals - The number of decimal places for the target currency.
 * @param {number} params.toCurrency.mid - The conversion rate from the source currency to the target currency.
 * @param {boolean} [params.negative=false] - If true, prefixes the formatted price with a negative sign.
 * @returns {string} The formatted price string.
 */
const priceFormatter = ({ price, fromSmallestUnit = false, negative = false, fromCurrency, toCurrency, }) => {
    const ensuredDecimal = (0, exports.ensureDecimal)(price);
    const decimalValue = new decimal_js_1.default(fromSmallestUnit
        ? divideConvertBack(ensuredDecimal, fromCurrency.decimals)
        : (0, exports.convertBack)(ensuredDecimal, fromCurrency.decimals));
    const symbol = (toCurrency === null || toCurrency === void 0 ? void 0 : toCurrency.symbol) || fromCurrency.symbol;
    const decimals = (toCurrency === null || toCurrency === void 0 ? void 0 : toCurrency.decimals) || fromCurrency.decimals;
    const mid = (toCurrency === null || toCurrency === void 0 ? void 0 : toCurrency.mid) || 1;
    const convertedValue = decimalValue.times(mid);
    return (0, currency_js_1.default)(convertedValue.toNumber(), {
        symbol,
        pattern: `${negative ? '-' : ''}! #`,
        precision: convertedValue.isInteger() ? 0 : decimals,
    }).format();
};
exports.priceFormatter = priceFormatter;
const ensureDecimal = (value) => {
    return value instanceof decimal_js_1.default ? value : new decimal_js_1.default(value);
};
exports.ensureDecimal = ensureDecimal;
const convertBack = (value, decimals) => {
    const decimalPlaces = (0, exports.ensureDecimal)(decimals);
    if (decimalPlaces.isZero()) {
        return value.round().toNumber();
    }
    // If any input was Decimal, return Decimal, otherwise return number
    return value.toDecimalPlaces(decimalPlaces.toNumber()).toNumber();
};
exports.convertBack = convertBack;
const multiplyByDecimals = (value, decimals) => {
    return value.times(new decimal_js_1.default(10).pow(decimals));
};
exports.multiplyByDecimals = multiplyByDecimals;
/**
 * Divide a value by decimals and convert back to the original unit, but if `returnSmallestUnit` is true, return the
 * smallest unit instead or main unit.
 * @param value
 * @param decimals
 * @param returnSmallestUnit
 */
const divideConvertBackSmallestUnit = (value, decimals, returnSmallestUnit) => {
    return returnSmallestUnit
        ? (0, exports.convertBack)(value, 0)
        : divideConvertBack(value, decimals);
};
exports.divideConvertBackSmallestUnit = divideConvertBackSmallestUnit;
const divideByDecimals = (value, decimals) => {
    return value.dividedBy(new decimal_js_1.default(10).pow((0, exports.ensureDecimal)(decimals).toNumber()));
};
const divideConvertBack = (value, decimals) => {
    return (0, exports.convertBack)(divideByDecimals(value, decimals), decimals);
};
